/*
 * AutomatonIteration.h
 *
 *  Created on: 29. 11. 2014
 *	  Author: Tomas Pecka
 */

#ifndef AUTOMATON_ITERATION_H_
#define AUTOMATON_ITERATION_H_

#include <automaton/FSM/EpsilonNFA.h>

namespace automaton {

namespace transform {

/**
 * Iterates language given by automaton
 *  - For finite automaton A1, we create automaton L accepting L(A1)*
 */
class AutomatonIteration {
public:
	template < class SymbolType, class StateType >
	static automaton::NFA < SymbolType, StateType > iteration(const automaton::DFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static automaton::NFA < SymbolType, StateType > iteration(const automaton::NFA < SymbolType, StateType > & automaton);
};

template < class SymbolType, class StateType >
automaton::NFA < SymbolType, StateType > AutomatonIteration::iteration(const automaton::DFA < SymbolType, StateType > & automaton) {
	automaton::NFA < SymbolType, StateType > res(automaton);

	for(const auto& qf : res.getFinalStates())
		for(const auto& t : res.getTransitionsToState(qf))
			res.addTransition(t.first.first, t.first.second, res.getInitialState());

	StateType newInitialState = common::createUnique(automaton.getInitialState(), automaton.getStates());
	res.addState(newInitialState);
	res.setInitialState(newInitialState);
	res.addFinalState(newInitialState);

	for(const auto& t : automaton.getTransitionsFromState(automaton.getInitialState()))
		res.addTransition(newInitialState, t.first.second, t.second);

	return res;
}

template < class SymbolType, class StateType >
automaton::NFA < SymbolType, StateType > AutomatonIteration::iteration(const automaton::NFA < SymbolType, StateType > & automaton) {
	automaton::NFA < SymbolType, StateType > res(automaton);

	for(const auto& qf : res.getFinalStates())
		for(const auto& t : res.getTransitionsToState(qf))
			res.addTransition(t.first.first, t.first.second, res.getInitialState());

	StateType newInitialState = common::createUnique(automaton.getInitialState(), automaton.getStates());
	res.addState(newInitialState);
	res.setInitialState(newInitialState);
	res.addFinalState(newInitialState);

	for(const auto& t : automaton.getTransitionsFromState(automaton.getInitialState()))
		for (const StateType & toState : t.second )
			res.addTransition(newInitialState, t.first.second, toState);

	return res;
}

} /* namespace transform */

} /* namespace automaton */

#endif /* AUTOMATON_ITERATION_H_ */
