/*
 * Compaction.h
 *
 *  Created on: 2. 11. 2014
 *	  Author: Tomas Pecka
 */

#ifndef COMPACTION_H_
#define COMPACTION_H_

#include <stack>
#include <alib/tuple>

#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

namespace automaton {

namespace transform {

/**
 * Transforms FSM to CompactNFA
 */
class Compaction {
public:
	template < class SymbolType, class StateType >
	static automaton::CompactNFA < SymbolType, StateType > convert( const automaton::DFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static automaton::CompactNFA < SymbolType, StateType > convert( const automaton::NFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static automaton::CompactNFA < SymbolType, StateType > convert( const automaton::CompactNFA < SymbolType, StateType > & automaton);
};

template < class SymbolType, class StateType >
automaton::CompactNFA < SymbolType, StateType > Compaction::convert(const automaton::CompactNFA < SymbolType, StateType > & automaton) {
	return automaton;
}

template < class SymbolType, class StateType >
automaton::CompactNFA < SymbolType, StateType > Compaction::convert(const automaton::DFA < SymbolType, StateType > & automaton) {
	automaton::CompactNFA < SymbolType, StateType > res(automaton.getInitialState());
	res.setInputAlphabet(automaton.getInputAlphabet());

	ext::set<StateType> visited;

	std::stack<ext::tuple<StateType, StateType, SymbolType>> stack; // state x lastFork x symbol we used to go to that state
	for(const auto& transition: automaton.getTransitionsFromState(automaton.getInitialState()))
		stack.push(ext::make_tuple(transition.second, automaton.getInitialState(), transition.first.second));

	if(automaton.getFinalStates().count(automaton.getInitialState()) > 0)
		res.addFinalState(automaton.getInitialState());

	while(!stack.empty()) {
		StateType q = std::move(std::get<0>(stack.top()));
		StateType lastFork = std::move(std::get<1>(stack.top()));
		SymbolType symbol = std::move(std::get<2>(stack.top()));
		stack.pop();

		ext::vector < SymbolType > path { symbol };

		ext::range < typename ext::map < ext::pair < StateType, SymbolType >, StateType >::const_iterator > transitions;
		// only 1 child and nonfinal
		while((transitions = automaton.getTransitionsFromState(q)).size() == 1 && automaton.getFinalStates().count(q) == 0) {
			const std::pair<ext::pair<StateType, SymbolType>, StateType>& transition = * transitions.begin();
			path.push_back(transition.first.second);
			q = transition.second;
		}

		// fork or final state
		res.addState(q);

		if(automaton.getFinalStates().count(q))
			res.addFinalState(q);

		res.addTransition(lastFork, path, q);

		for(const std::pair<const ext::pair<StateType, SymbolType>, StateType>& transition : automaton.getTransitionsFromState(q))
			if(visited.insert(transition.second).second)
				stack.push(ext::make_tuple(transition.second, q, transition.first.second));

	}

	return res;
}

template < class SymbolType, class StateType >
automaton::CompactNFA < SymbolType, StateType > Compaction::convert(const automaton::NFA < SymbolType, StateType > & automaton) {
	automaton::CompactNFA < SymbolType, StateType > res(automaton.getInitialState());
	// TODO
	return res;
}

} /* namespace transform */

} /* namespace automaton */

#endif /* COMPACTION_H_ */
