/*
 * RightRegularEquationSolver.h
 *
 *  Created on: 4. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef RIGHT_REGULAR_EQUATIONS_SOLVER_H_
#define RIGHT_REGULAR_EQUATIONS_SOLVER_H_

#include "RegularEquationSolver.h"
#include "regexp/unbounded/UnboundedRegExpElements.h"

#include <regexp/simplify/RegExpOptimize.h>

namespace equations {

template < class SymbolType >
class RightRegularEquationSolver : public RegularEquationSolver < SymbolType > {
	/**
	 * @copydoc RegularEquationSolver::eliminate(void)
	 */
	virtual regexp::UnboundedRegExp < SymbolType > eliminate( void );

};

template < class SymbolType >
regexp::UnboundedRegExp < SymbolType > RightRegularEquationSolver < SymbolType >::eliminate(void) {
	for(auto itA = this->nonterminalSymbolsByDepth.rbegin(); itA != this->nonterminalSymbolsByDepth.rend(); ++ itA) {
		const SymbolType& a = * itA;

		/*
		 * Apply Arden's Lemma
		 * A = 0A + 1B + 2C
		 * => A = 0*1B + 0*2C
		 */
		regexp::UnboundedRegExpIteration < SymbolType > loop(std::move(this->equationTransition.find(std::make_pair(a, a))->second));
		regexp::simplify::RegExpOptimize::optimize(loop);

		// for all transitions from A apply Arden's Lemma
		for(auto itB = std::next(itA) ; itB != this->nonterminalSymbolsByDepth.rend(); ++ itB) {
			const SymbolType& b = * itB;
			regexp::UnboundedRegExpConcatenation < SymbolType > concat;
			concat.appendElement(loop);
			concat.appendElement(std::move(this->equationTransition.find(std::make_pair(a,b))->second));
			regexp::UnboundedRegExpAlternation < SymbolType > alt;
			alt.appendElement(std::move(concat));
			this->equationTransition.find(std::make_pair(a, b))->second = std::move(alt);
			regexp::simplify::RegExpOptimize::optimize(this->equationTransition.find(std::make_pair(a, b))->second);
		}

		{
			regexp::UnboundedRegExpConcatenation < SymbolType > concat;
			concat.appendElement(std::move(loop));
			concat.appendElement(std::move(this->equationFinal.find(a)->second));
			regexp::UnboundedRegExpAlternation < SymbolType > alt;
			alt.appendElement(std::move(concat));
			this->equationFinal.find(a)->second = std::move(alt);
			regexp::simplify::RegExpOptimize::optimize(this->equationFinal.find(a)->second);
		}

		/*
		 * eliminate A from rest of the equations using this pattern:
		 * B->C = B->C + concatenate(B->A, A->C)
		 */
		for(auto itB = std::next(itA); itB != this->nonterminalSymbolsByDepth.rend(); ++ itB) {
			const SymbolType& b = * itB;

			for(auto itC = std::next(itA); itC != this->nonterminalSymbolsByDepth.rend(); ++ itC) {
				const SymbolType& c = * itC;

				regexp::UnboundedRegExpConcatenation < SymbolType > concat;
				concat.appendElement(this->equationTransition.find(std::make_pair(b, a))->second);
				concat.appendElement(this->equationTransition.find(std::make_pair(a, c))->second);
				regexp::UnboundedRegExpAlternation < SymbolType > alt;
				alt.appendElement(std::move(this->equationTransition.find(std::make_pair(b, c))->second));
				alt.appendElement(std::move(concat));
				this->equationTransition.find(std::make_pair(b, c))->second = std::move(alt);
				regexp::simplify::RegExpOptimize::optimize(this->equationTransition.find(std::make_pair(b, c))->second);
			}

			{
				regexp::UnboundedRegExpConcatenation < SymbolType > concat;
				concat.appendElement(std::move(this->equationTransition.find(std::make_pair(b, a))->second));
				concat.appendElement(this->equationFinal.find(a)->second);
				regexp::UnboundedRegExpAlternation < SymbolType > alt;
				alt.appendElement(std::move(this->equationFinal.find(b)->second));
				alt.appendElement(std::move(concat));
				this->equationFinal.find(b)->second = std::move(alt);
				regexp::simplify::RegExpOptimize::optimize(this->equationFinal.find(b)->second);
			}
		}
	}

	return regexp::UnboundedRegExp < SymbolType > (regexp::simplify::RegExpOptimize::optimize(regexp::UnboundedRegExpStructure < SymbolType > (std::move( this->equationFinal.find(*this->nonterminalSymbolsByDepth.begin())->second))));
}

} /* namespace equations */

#endif /* RIGHT_REGULAR_EQUATIONS_SOLVER_H_ */
