/*
 * Bool.h
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_BOOL_H_
#define PRIMITIVE_BOOL_H_

#include "PrimitiveBase.h"

namespace primitive {

/**
 * \brief
 * Represents a wrapper of a boolean datatype.
 */
class Bool final : public PrimitiveBase {
	/**
	 * \brief
	 * The stored boolean value.
	 */
	bool m_data;

public:
	/**
	 * \brief
	 * Creates new instance holding provided boolean value.
	 *
	 * \param data the boolean value to wrap
	 */
	explicit Bool ( bool data );

	/**
	 * @copydoc primitive::PrimitiveBase::clone ( ) const &
	 */
	virtual PrimitiveBase * clone ( ) const & override;

	/**
	 * @copydoc primitive::PrimitiveBase::clone ( ) &&
	 */
	virtual PrimitiveBase * clone ( ) && override;

	/**
	 * Getter of the stored boolean value.
	 *
	 * \return the stored boolean value
	 */
	bool getData ( ) const;

	/**
	 * \brief
	 * Cast operator to the boolean type. The result of the cast is the stored boolean value.
	 */
	explicit operator bool ( ) const;

	/**
	 * Setter of a new value.
	 *
	 * \param data a new boolean value to store.
	 */
	void setData ( bool data );

	/**
	 * @copydoc base::CommonBase < ObjectBase >::compare ( const ObjectBase & ) const
	 */
	virtual int compare ( const ObjectBase & other ) const override {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	/**
	 * The actual compare method
	 *
	 * \param other the other instance
	 *
	 * \returns the actual relation between two by type same primitives
	 */
	int compare ( const Bool & other ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator >> ( std::ostream & ) const
	 */
	virtual void operator >>( std::ostream & ) const override;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator std::string ( ) const
	 */
	virtual explicit operator std::string ( ) const override;

	/**
	 * @copydoc primitive::PrimitiveBase::inc ( ) &&
	 */
	virtual PrimitiveBase * inc ( ) && override;
};

} /* namespace primitive */

#endif /* PRIMITIVE_BOOL_H_ */
