/*
 * String.h
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_STRING_H_
#define PRIMITIVE_STRING_H_

#include "PrimitiveBase.h"
#include <alib/string>

namespace primitive {

/**
 * \brief
 * Represents a wrapper of a string datatype.
 */
class String final : public PrimitiveBase {
	/**
	 * \brief
	 * The stored string value.
	 */
	std::string m_data;

public:
	/**
	 * \brief
	 * Creates new instance holding provided string value.
	 *
	 * \param data the string value to wrap
	 */
	explicit String ( std::string data );

	/**
	 * \brief
	 * Creates new instance holding provided c-string value.
	 *
	 * \param data the string value to wrap
	 */
	explicit String ( const char * data );

	/**
	 * @copydoc primitive::PrimitiveBase::clone ( ) const &
	 */
	virtual PrimitiveBase * clone ( ) const &;

	/**
	 * @copydoc primitive::PrimitiveBase::clone ( ) &&
	 */
	virtual PrimitiveBase * clone ( ) &&;

	/**
	 * Getter of the stored string value.
	 *
	 * \return the stored string value
	 */
	const std::string & getData ( ) const &;

	/**
	 * Getter of the stored string value.
	 *
	 * \return the stored string value
	 */
	std::string && getData ( ) &&;

	/**
	 * Setter of a new value.
	 *
	 * \param data a new string value to store.
	 */
	void setData ( std::string data );

	/**
	 * @copydoc base::CommonBase < ObjectBase >::compare ( const ObjectBase & ) const
	 */
	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	/**
	 * The actual compare method
	 *
	 * \param other the other instance
	 *
	 * \returns the actual relation between two by type same primitives
	 */
	virtual int compare ( const String & other ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator >> ( std::ostream & ) const
	 */
	virtual void operator >>( std::ostream & ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator std::string ( ) const
	 */
	virtual explicit operator std::string ( ) const;

	/**
	 * @copydoc primitive::PrimitiveBase::inc ( ) &&
	 */
	virtual PrimitiveBase * inc ( ) &&;
};

} /* namespace primitive */

#endif /* PRIMITIVE_STRING_H_ */
