#include <alib/list>
#include <alib/type_traits>
#include <alib/set>

#include "ObjectTest.h"

#include "object/Object.h"
#include "object/ObjectFactory.h"
#include <primitive/Unsigned.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( ObjectTest, "object" );
CPPUNIT_TEST_SUITE_REGISTRATION ( ObjectTest );

void ObjectTest::setUp ( ) {
}

void ObjectTest::tearDown ( ) {
}

void ObjectTest::testProperties ( ) {
	CPPUNIT_ASSERT ( std::is_nothrow_move_constructible < object::Object >::value );
	CPPUNIT_ASSERT ( std::is_move_constructible < object::Object >::value && std::is_move_assignable < object::Object >::value );

	object::Object tmp1 { primitive::Unsigned ( 1 ) };
	object::Object tmp2 { primitive::Unsigned ( 2 ) };

	std::swap ( tmp1, tmp2 );

	CPPUNIT_ASSERT ( tmp1 == object::Object ( primitive::Unsigned ( 2 ) ) );
	CPPUNIT_ASSERT ( tmp2 == object::Object ( primitive::Unsigned ( 1 ) ) );

}

void ObjectTest::testConstruction ( ) {
	object::Object tmp1 = object::ObjectFactory::make ( 1 );
	object::Object tmp2 = object::ObjectFactory::make ( ext::variant < int, std::string > ( 1 ) );
	object::Object tmp3 = object::ObjectFactory::make ( ext::variant < int, ext::variant < std::string, int > > ( ext::variant < std::string, int > ( 1 ) ) );

	CPPUNIT_ASSERT ( tmp1 == tmp2 );
	CPPUNIT_ASSERT ( tmp1 == tmp3 );

	object::Object tmp4 = object::Object ( object::AnyObject < ext::set < int > > ( ext::set < int > { } ) );
	object::Object tmp5 = object::ObjectFactory::make ( ext::set < int > { } );
	object::Object tmp6 = object::ObjectFactory::make ( object::AnyObject < ext::set < int > > ( ext::set < int > { } ) );

	std::cout << tmp6 << std::endl;

	CPPUNIT_ASSERT ( tmp4 == tmp5 );
	CPPUNIT_ASSERT ( tmp4 == tmp6 );
}
