/*
 * TreeAuxiliary.h
 *
 *  Created on: May 5, 2016
 *      Author: Jan Travnicek
 */

#ifndef TREE_AUXILIARY_H_
#define TREE_AUXILIARY_H_

#include <alib/set>
#include <alib/tree>
#include <alib/list>
#include <alib/algorithm>
#include <stack>
#include <common/ranked_symbol.hpp>

namespace tree {

/**
 * Parser used to get tree from XML parsed into list of Tokens.
 */
class TreeAuxiliary {
	template < class SymbolType, class RankType >
	static void postfixToPrefixInt ( const ext::vector < common::ranked_symbol < SymbolType, RankType > > & src, ext::vector < common::ranked_symbol < SymbolType, RankType > > & dest, unsigned & readIndex );

public:
	template < class SymbolType, class RankType >
	static ext::set < SymbolType > unrankSymbols ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabet );
	template < class SymbolType, class RankType >
	static ext::set < common::ranked_symbol < SymbolType, RankType > > computeBars ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabet, const SymbolType & barBase );

	template < class SymbolType, class RankType >
	static ext::tree < common::ranked_symbol < SymbolType, RankType > > unrankedToRanked ( const ext::tree < SymbolType > & from );
	template < class SymbolType, class RankType >
	static ext::tree < SymbolType > rankedToUnranked ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & from );

	template < class SymbolType, class RankType >
	static ext::tree < common::ranked_symbol < SymbolType, RankType > > postfixToTree (const ext::vector < common::ranked_symbol < SymbolType, RankType > > & from);

	template < class SymbolType, class RankType >
	static ext::vector < common::ranked_symbol < SymbolType, RankType > > postfixToPrefix ( const ext::vector < common::ranked_symbol < SymbolType, RankType > > & src );

	template < class SymbolType, class RankType >
	static ext::vector < common::ranked_symbol < SymbolType, RankType > > treeToPrefix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree );
	template < class SymbolType >
	static ext::vector < SymbolType > treeToPrefix ( const ext::tree < SymbolType > & tree, const SymbolType & bar );
	template < class SymbolType, class RankType >
	static ext::vector < common::ranked_symbol < SymbolType, RankType > > treeToPrefix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, const SymbolType & barBase );
	template < class SymbolType, class RankType >
	static ext::vector < common::ranked_symbol < SymbolType, RankType > > treeToPrefix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, const common::ranked_symbol < SymbolType, RankType > & subtreeWildcard, const SymbolType & barBase, const common::ranked_symbol < SymbolType, RankType > & variablesBar );
	template < class SymbolType, class RankType >
	static ext::vector < common::ranked_symbol < SymbolType, RankType > > treeToPrefix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, const common::ranked_symbol < SymbolType, RankType > & subtreeWildcard, const ext::set < common::ranked_symbol < SymbolType, RankType > > & nonlinearVariables, const SymbolType & barBase, const common::ranked_symbol < SymbolType, RankType > & variablesBar );

	template < class SymbolType, class RankType >
	static ext::vector < common::ranked_symbol < SymbolType, RankType > > treeToPostfix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree );
};

template < class SymbolType, class RankType >
ext::set < SymbolType > TreeAuxiliary::unrankSymbols ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabet ) {
	return ext::transform < SymbolType > ( alphabet, [&] ( const common::ranked_symbol < SymbolType, RankType > & symbol ) {
			return symbol.getSymbol ( );
	} );
}

template < class SymbolType, class RankType >
ext::set < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::computeBars ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabet, const SymbolType & barBase ) {
	return ext::transform < common::ranked_symbol < SymbolType, RankType > > ( alphabet, [&] ( const common::ranked_symbol < SymbolType, RankType > & symbol ) {
			return common::ranked_symbol < SymbolType, RankType > ( barBase, symbol.getRank ( ) );
	} );
}

template < class SymbolType, class RankType >
ext::tree < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::unrankedToRanked ( const ext::tree < SymbolType > & tree ) {
	ext::vector < ext::tree < common::ranked_symbol < SymbolType, RankType > > > children;

	for ( const ext::tree < SymbolType > & child : tree ) {
		children.push_back ( unrankedToRanked < SymbolType, RankType > ( child ) );
	}

	unsigned size = children.size ( );
	return ext::tree < common::ranked_symbol < SymbolType, RankType > > ( common::ranked_symbol < SymbolType, RankType > ( tree.getData ( ), RankType ( size ) ), std::move ( children ) );
}

template < class SymbolType, class RankType >
ext::tree < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::postfixToTree ( const ext::vector < common::ranked_symbol < SymbolType, RankType > > & from ) {

	std::stack < ext::tree < common::ranked_symbol < SymbolType, RankType > > > tree_stack;

	for ( auto x : from ) {
		if ( ( unsigned ) x.getRank ( ) == 0 ) {
			tree_stack.push ( ext::tree < common::ranked_symbol < SymbolType, RankType > > ( common::ranked_symbol < SymbolType, RankType > ( x.getSymbol ( ), x.getRank ( ) ), ext::vector < ext::tree < common::ranked_symbol < SymbolType, RankType > > > ( ) ) );
		} else {
			unsigned children_size = ( unsigned ) x.getRank ( );
			ext::list < ext::tree < common::ranked_symbol < SymbolType, RankType > > > childrenList;

			for ( unsigned i = 0; i < children_size; ++i ) {
				ext::tree < common::ranked_symbol < SymbolType, RankType > > child = tree_stack.top ( );
				tree_stack.pop ( );
				childrenList.push_front ( child );
			}

			ext::vector < ext::tree < common::ranked_symbol < SymbolType, RankType > > > children {
				std::begin ( childrenList ), std::end ( childrenList )
			};
			tree_stack.push ( ext::tree < common::ranked_symbol < SymbolType, RankType > > ( common::ranked_symbol < SymbolType, RankType > ( x.getSymbol ( ), x.getRank ( ) ), std::move ( children ) ) );
		}
	}

	return tree_stack.top ( );
}


template < class SymbolType, class RankType >
ext::tree < SymbolType > TreeAuxiliary::rankedToUnranked ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree ) {
	ext::vector < ext::tree < SymbolType > > children;

	for ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & child : tree ) {
		children.push_back ( rankedToUnranked < SymbolType, RankType > ( child ) );
	}

	return ext::tree < SymbolType > ( tree.getData ( ).getSymbol ( ), std::move ( children ) );
}

template < class SymbolType, class RankType >
ext::vector < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::postfixToPrefix ( const ext::vector < common::ranked_symbol < SymbolType, RankType > > & src ) {
	unsigned readIndex = src.size ( ) - 1;

	ext::vector < common::ranked_symbol < SymbolType, RankType > > dest;
	postfixToPrefixInt ( src, dest, readIndex );

	std::reverse ( dest.begin ( ), dest.end ( ) );

	return dest;
}

template < class SymbolType, class RankType >
void TreeAuxiliary::postfixToPrefixInt ( const ext::vector < common::ranked_symbol < SymbolType, RankType > > & src, ext::vector < common::ranked_symbol < SymbolType, RankType > > & dest, unsigned & readIndex ) {
	unsigned root = readIndex --;
	unsigned rank = ( unsigned ) src [ root ].getRank ( );

	for ( unsigned i = 0; i < rank; ++ i )
		postfixToPrefixInt ( src, dest, readIndex );

	dest.push_back ( src [ root ] );
}

template < class SymbolType, class RankType >
ext::vector < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::treeToPrefix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree ) {
	ext::vector < common::ranked_symbol < SymbolType, RankType > > res;

	for ( typename ext::tree < common::ranked_symbol < SymbolType, RankType > >::const_prefix_iterator iter = tree.prefix_begin ( ); iter != tree.prefix_end ( ); ++iter )
		res.push_back ( * iter );

	return res;
}

template < class SymbolType >
ext::vector < SymbolType > TreeAuxiliary::treeToPrefix ( const ext::tree < SymbolType > & tree, const SymbolType & bar ) {
	ext::vector < SymbolType > res;

	for ( typename ext::tree < SymbolType >::const_structure_iterator iter = tree.structure_begin ( ); iter != tree.structure_end ( ); ++iter )
		if ( iter.getVirtual ( ) )
			res.push_back ( bar );
		else
			res.push_back ( * iter );

	return res;
}

template < class SymbolType, class RankType >
ext::vector < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::treeToPrefix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, const SymbolType & barBase ) {
	ext::vector < common::ranked_symbol < SymbolType, RankType > > res;

	for ( typename ext::tree < common::ranked_symbol < SymbolType, RankType > >::const_structure_iterator iter = tree.structure_begin ( ); iter != tree.structure_end ( ); ++iter )
		if ( iter.getVirtual ( ) )
			res.push_back ( common::ranked_symbol < SymbolType, RankType > ( barBase, iter->getRank ( ) ) );
		else
			res.push_back ( * iter );

	return res;
}

template < class SymbolType, class RankType >
ext::vector < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::treeToPrefix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, const common::ranked_symbol < SymbolType, RankType > & subtreeWildcard, const SymbolType & barBase, const common::ranked_symbol < SymbolType, RankType > & variablesBar ) {
	ext::vector < common::ranked_symbol < SymbolType, RankType > > res;

	for ( typename ext::tree < common::ranked_symbol < SymbolType, RankType > >::const_structure_iterator iter = tree.structure_begin ( ); iter != tree.structure_end ( ); ++iter )
		if ( iter.getVirtual ( ) )
			if ( ( * iter == subtreeWildcard ) )
				res.push_back ( variablesBar );
			else
				res.push_back ( common::ranked_symbol < SymbolType, RankType > ( barBase, iter->getRank ( ) ) );
		else
			res.push_back ( * iter );

	return res;
}

template < class SymbolType, class RankType >
ext::vector < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::treeToPrefix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, const common::ranked_symbol < SymbolType, RankType > & subtreeWildcard, const ext::set < common::ranked_symbol < SymbolType, RankType > > & nonlinearVariables, const SymbolType & barBase, const common::ranked_symbol < SymbolType, RankType > & variablesBar ) {
	ext::vector < common::ranked_symbol < SymbolType, RankType > > res;

	for ( typename ext::tree < common::ranked_symbol < SymbolType, RankType > >::const_structure_iterator iter = tree.structure_begin ( ); iter != tree.structure_end ( ); ++iter )
		if ( iter.getVirtual ( ) )
			if ( ( * iter == subtreeWildcard ) || nonlinearVariables.count ( * iter ) )
				res.push_back ( variablesBar );
			else
				res.push_back ( common::ranked_symbol < SymbolType, RankType > ( barBase, iter->getRank ( ) ) );
		else
			res.push_back ( * iter );

	return res;
}

template < class SymbolType, class RankType >
ext::vector < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::treeToPostfix ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree ) {
	ext::vector < common::ranked_symbol < SymbolType, RankType > > res;

	for ( typename ext::tree < common::ranked_symbol < SymbolType, RankType > >::const_postfix_iterator iter = tree.postfix_begin ( ); iter != tree.postfix_end ( ); ++iter )
		res.push_back ( * iter );

	return res;
}

} /* namespace tree */

#endif /* TREE_AUXILIARY_H_ */
