#include <alib/list>
#include "SymbolTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "alphabet/Symbol.h"
#include "alphabet/BlankSymbol.h"
#include "alphabet/StartSymbol.h"
#include "alphabet/EndSymbol.h"
#include "alphabet/RankedSymbol.h"
#include "alphabet/BarSymbol.h"

#include <common/DefaultSymbolType.h>

#include "factory/XmlDataFactory.hpp"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( SymbolTest, "alphabet" );
CPPUNIT_TEST_SUITE_REGISTRATION ( SymbolTest );

void SymbolTest::setUp ( ) {
}

void SymbolTest::tearDown ( ) {
}

void SymbolTest::testOrder ( ) {
	alphabet::BlankSymbol bs;
	alphabet::StartSymbol ss;
	alphabet::EndSymbol es;

	alphabet::RankedSymbol < > rs1 ( DefaultSymbolType ( alphabet::BarSymbol { } ), DefaultRankType ( 0 ) );
	alphabet::RankedSymbol < > rs2 ( DefaultSymbolType ( alphabet::BarSymbol { } ), DefaultRankType ( 0 ) );

	CPPUNIT_EXCLUSIVE_OR ( bs < ss, ss < bs );
	CPPUNIT_EXCLUSIVE_OR ( bs < es, es < bs );

	CPPUNIT_ASSERT ( rs1 == rs2 );

	// CPPUNIT_ASSERT(es < ls); //End symbol typically denoted by '$' is lexicographically smaller than any other symbol
	// CPPUNIT_ASSERT(ls < ss); //Start symbol typically denoted by '^' is lexicographically bigger than any other symbol
}

namespace alphabet {

template < >
inline int BarSymbol::instance < int > ( ) {
	return -1;
}

} /* namespace alphabet */

void SymbolTest::testDefaultValues ( ) {
	CPPUNIT_ASSERT ( alphabet::BarSymbol::instance < int > ( ) == -1 );
	CPPUNIT_ASSERT ( alphabet::BarSymbol::instance < char > ( ) == '|');
	std::cout << alphabet::BarSymbol::instance < std::string > ( ) << std::endl;
	CPPUNIT_ASSERT ( alphabet::BarSymbol::instance < std::string > ( ) == "|");
	CPPUNIT_ASSERT ( alphabet::BarSymbol::instance < alphabet::BarSymbol > ( ) == alphabet::BarSymbol ( ) );
	CPPUNIT_ASSERT ( alphabet::BarSymbol::instance < alphabet::Symbol > ( ) == alphabet::Symbol ( alphabet::BarSymbol ( ) ) );
	CPPUNIT_ASSERT ( alphabet::BarSymbol::instance < object::Object > ( ) == object::Object ( alphabet::BarSymbol ( ) ) );

}
