/*
 * list.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __LIST_HPP_
#define __LIST_HPP_

#include <list>
#include <ostream>
#include <sstream>
#include <string>

#include "compare.hpp"
#include "allocFix.hpp"

namespace ext {

template < class T, class Alloc = std::allocator < T > >
class list : public std::list < T, Alloc >, AllocFix < Alloc > {
public:
	using std::list < T, Alloc >::list;
	using std::list < T, Alloc >::operator =;
#ifndef __clang__
	list ( ) = default;

	list ( const list & other ) = default;

	list ( list && other ) = default;

	list & operator = ( list && other ) = default;

	list & operator = ( const list & other ) = default;
#endif
};

template< class T, class ... Ts >
std::ostream& operator<<(std::ostream& out, const ext::list<T, Ts ... >& list) {
	out << "[";

	bool first = true;
	for(const T& item : list) {
		if(!first) out << ", ";
		first = false;
		out << item;
	}

	out << "]";
	return out;
}

template < class T, class ... Ts >
struct compare < ext::list < T, Ts ... > > {
	int operator ( ) ( const ext::list < T, Ts ... > & first, const ext::list < T, Ts ... > & second) const {
		if(first.size() < second.size()) return -1;
		if(first.size() > second.size()) return 1;

		static compare<typename std::decay < T >::type > comp;
		for(auto iterF = first.begin(), iterS = second.begin(); iterF != first.end(); ++iterF, ++iterS) {
			int res = comp(*iterF, *iterS);
			if(res != 0) return res;
		}
		return 0;
	}
};

template < class T, class ... Ts >
std::string to_string ( const ext::list < T, Ts ... > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

} /* namespace ext */

#endif /* __LIST_HPP_ */
