/*
 * utility.hpp
 *
 * Created on: Apr 7, 2016
 * Author: Jan Travnicek
 */

#ifndef __UTILITY_HPP_
#define __UTILITY_HPP_

#include <utility>
#include <cstddef>

#include "type_traits.hpp"

namespace ext {

template<class T>
struct SizeOf {
	static const size_t size = sizeof(T);
};

template<>
struct SizeOf<void> {
	static const size_t size = 1;
};

template<class T>
struct AlignOf {
	static const size_t align = alignof(T);
};

template<>
struct AlignOf<void> {
	static const size_t align = 1;
};

template < class T, typename std::enable_if < ! ext::has_clone < T >::value >::type * = nullptr >
inline T move_copy ( const T & param ) {
	return T ( param );
}

template < class T >
struct rvalue_ref {
	T * holder;

	explicit rvalue_ref ( T * ptr ) : holder ( ptr ) {

	}

	template < class U >
	rvalue_ref ( rvalue_ref < U > && other ) : holder ( other.holder ) {
		other.holder = nullptr;
	}

	rvalue_ref ( const rvalue_ref & ) = delete;

	rvalue_ref ( rvalue_ref && other) : holder( other.holder ) {
		other.holder = nullptr;
	};

	rvalue_ref & operator = ( const rvalue_ref & ) = delete;

	rvalue_ref & operator = ( rvalue_ref && other ) {
		using std::swap;

		swap(holder, other.holder);

		return *this;
	}

	~rvalue_ref ( ) {
		delete holder;
	}

	T * operator ->( ) && {
		return holder;
	}

	T * operator ->( ) const & {
		return holder;
	}

	operator T && ( ) {
		return std::move ( * holder );
	}

	const T & get ( ) const {
		return * holder;
	}
};

template < class T, typename std::enable_if < ext::has_clone < T >::value >::type * = nullptr >
inline rvalue_ref < T > move_copy ( const T & param ) {
	return rvalue_ref < T > { param.clone ( ) };
}

 // TODO remove after switching to c++17 (as_const)
template < class T >
inline const T & make_const ( T & ptr ) {
	return static_cast < const T & > ( ptr );
}

template < class T >
inline const T & make_const ( const T & ptr ) {
	return ptr;
}

} /* namespace ext */

#endif /* __UTILITY_HPP_ */
