#include "VariantTest.h"
#include <alib/variant>
#include <alib/set>
#include <alib/algorithm>
#include <alib/map>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( VariantTest, "bits" );
CPPUNIT_TEST_SUITE_REGISTRATION( VariantTest );

void VariantTest::setUp() {
}

void VariantTest::tearDown() {
}

void VariantTest::testVariant() {
	ext::variant<int, std::string, VariantTest::test> d(10);
	CPPUNIT_ASSERT( d.is<int>() );
	CPPUNIT_ASSERT( d.get<int>() == 10 );

	std::string str = "abcde";
	d.set<std::string>(str);
	CPPUNIT_ASSERT( d.is<std::string>() );
	CPPUNIT_ASSERT( d.get<std::string>() == str );

	d.set<VariantTest::test>(VariantTest::test());
	*(d.get<VariantTest::test>().holder) = 42;

	VariantTest::test tmp;
	*(tmp.holder) = 42;
	CPPUNIT_ASSERT( d.is<VariantTest::test>() );
	CPPUNIT_ASSERT( d.get<VariantTest::test>() == tmp );

	ext::variant<int, std::string, VariantTest::test> e(d);
	CPPUNIT_ASSERT( e.get<VariantTest::test>() == tmp );

	*(e.get<VariantTest::test>().holder) = 43;

	CPPUNIT_ASSERT( (d < e) == true );
	CPPUNIT_ASSERT( (e < d) == false );

	std::swap ( d, e );

	CPPUNIT_ASSERT( (d < e) == false );
	CPPUNIT_ASSERT( (e < d) == true );

	std::swap ( d, e );

	CPPUNIT_ASSERT( (e == d) == false );

	d = ext::variant<int, std::string, VariantTest::test>(0);
	d = e;

	CPPUNIT_ASSERT( *(d.get<VariantTest::test>().holder) == 43 );

	CPPUNIT_ASSERT( (d < e) == false );
	CPPUNIT_ASSERT( (e < d) == false );

	CPPUNIT_ASSERT( (e == d) == true );
}

void VariantTest::testVariantSet() {
	VariantTest::test tmp;
	*(tmp.holder) = 42;

	std::string str = "abcde";

	ext::variant<int, std::string, VariantTest::test> d(VariantTest::test {});
	*(d.get<VariantTest::test>().holder) = 42;
	ext::variant<int, std::string, VariantTest::test> e(d);

	ext::variant<int, std::string, VariantTest::test> f(5);

	ext::variant<int, std::string, VariantTest::test> g(str);

	ext::set<ext::variant<int, std::string, VariantTest::test>> testSet;
	testSet.insert(d);
	testSet.insert(e);
	testSet.insert(f);
	testSet.insert(g);

	CPPUNIT_ASSERT( testSet.size() == 3 );

	for(ext::set<ext::variant<int, std::string, VariantTest::test>>::const_iterator iter = testSet.begin(); iter != testSet.end(); iter++) {
		if(iter->is<int>())
			CPPUNIT_ASSERT( iter->get<int>() == 5);
		if(iter->is<std::string>())
			CPPUNIT_ASSERT( iter->get<std::string>() == str );
		if(iter->is<VariantTest::test>())
			CPPUNIT_ASSERT( iter->get<VariantTest::test>() == tmp );
	}

	ext::variant<VariantTest::test2, std::string, VariantTest::test> h("aa");
	CPPUNIT_ASSERT( h.is<std::string>() );

	ext::map<ext::variant<std::string, int>, int> testMap;
	testMap.insert(std::make_pair(ext::variant<std::string, int> {"aa"}, 10));

	CPPUNIT_ASSERT( testMap.size() == 1 );
	CPPUNIT_ASSERT( testMap.find(ext::variant<std::string, int> {"aa"}) != testMap.end() );
	CPPUNIT_ASSERT( testMap.find(ext::variant<std::string, int> {10}) == testMap.end() );

	testMap.insert(std::make_pair(ext::variant<std::string, int> {"ab"}, 11));
	testMap.insert(std::make_pair(ext::variant<std::string, int> {3}, 13));

	CPPUNIT_ASSERT( testMap.find(ext::variant<std::string, int> {"aa"}) != testMap.end() );
	CPPUNIT_ASSERT( testMap.find(ext::variant<std::string, int> {10}) == testMap.end() );
}

void VariantTest::testVariantSet2() {
	ext::set<ext::variant<std::string, int>> s, t, u;
	s.insert(std::string { "aa" } );
	s.insert(123);

	std::set_union(s.begin(), s.end(), t.begin(), t.end(), std::inserter(u, u.begin()));

	CPPUNIT_ASSERT( s.size() == u.size());

/*	ext::variant < void, ext::pair < int, int > > var;
	var.get < int > ( );*///should not compile -- correct behavior
}

void VariantTest::testVariantVoid() {
	ext::variant<void, int> i(1);
	ext::variant<void, int> v;

	CPPUNIT_ASSERT ( v.is < void > ( ) );
	CPPUNIT_ASSERT ( i.is < int > ( ) );

	i.set();
	CPPUNIT_ASSERT ( i.is < void > ( ) );

	ext::variant<void, int> copy = i;

	CPPUNIT_ASSERT ( copy == v );

	v = 10;

	CPPUNIT_ASSERT ( v.get<int>() == 10 );

	ext::variant<int, void> v2;
	CPPUNIT_ASSERT ( v2.is < void > ( ) );
}

void VariantTest::testVariantDefault() {
	ext::variant<int, std::string, void> v1 = ext::variant<int, std::string, void>::from<std::string>();

	CPPUNIT_ASSERT ( v1.get<std::string>() == "");

	ext::variant<int, std::string, void> v2 = ext::variant<int, std::string, void>::from<void>();

	CPPUNIT_ASSERT ( v2.is<void> ( ) );

	ext::variant<int, std::string, void> v3 = ext::variant<int, std::string, void>::from<int>();

	CPPUNIT_ASSERT ( v3.get<int> ( ) == 0 );
}

void VariantTest::testVariantSameTypes() {
	ext::variant < int, int, char > v1 ( 1 );

	CPPUNIT_ASSERT ( v1.get < int > ( ) == 1 );

	v1.set < char > ( 'a' );

	CPPUNIT_ASSERT ( v1.get < char > ( ) == 'a' );

	v1.set < int > ( 'a' );

	CPPUNIT_ASSERT ( v1.is < int > ( ) );
}

void VariantTest::testSizeT() {
	ext::variant < size_t, int, void > a ( ( size_t ) 1 );
	ext::variant < size_t, int, void > b ( 1 );
	ext::variant < size_t, int, void > c;

	CPPUNIT_ASSERT ( a.is < size_t > ( ) );
	CPPUNIT_ASSERT ( b.is < int > ( ) );
	CPPUNIT_ASSERT ( c.is < void > ( ) );
}

void VariantTest::testMoveSemantics() {
	CPPUNIT_ASSERT ( ( std::is_nothrow_move_constructible < ext::variant < void, int, size_t > >::value ) );
	CPPUNIT_ASSERT ( ( std::is_move_constructible < ext::variant < void, int, size_t > >::value && std::is_move_assignable < ext::variant < void, int, size_t > >::value ) );
}

void VariantTest::testSetVariant ( ) {
	std::set < int > a;
	ext::variant < void, std::set < int > > v = a;
	ext::variant < void, std::set < int > > v2 = a;
	v = std::move ( v2 );
	v.get < std::set < int > > ( ).insert ( 1 );
}

void VariantTest::testVariantBuilder ( ) {
	bool same;
	same = std::is_same < ext::variant_builder < int >::type, int >::value;
	CPPUNIT_ASSERT ( same );

	same = std::is_same < ext::variant_builder < int, int, int >::type, int >::value;
	CPPUNIT_ASSERT ( same );

	same = std::is_same < ext::variant_builder < double, int, char >::type, ext::variant < double, int, char > >::value;
	CPPUNIT_ASSERT ( same );

	same = std::is_same < ext::variant_builder < double, int, char, char, int, double >::type, ext::variant < char, int, double > >::value;
	CPPUNIT_ASSERT ( same );
}
