#include <alib/list>
#include "GrammarTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "grammar/Unrestricted/UnrestrictedGrammar.h"
#include "grammar/Unrestricted/ContextPreservingUnrestrictedGrammar.h"

#include "grammar/Regular/RightRG.h"
#include "grammar/Regular/RightLG.h"

#include "grammar/ContextFree/LG.h"
#include "grammar/ContextFree/EpsilonFreeCFG.h"
#include "grammar/ContextFree/CFG.h"
#include "grammar/ContextFree/CNF.h"
#include "grammar/ContextFree/GNF.h"

#include "grammar/ContextSensitive/CSG.h"
#include "grammar/ContextSensitive/NonContractingGrammar.h"

#include <grammar/Grammar.h>

#include "factory/XmlDataFactory.hpp"
#include "factory/StringDataFactory.hpp"

#include "grammar/string/Regular/RightRG.h"
#include "primitive/string/Character.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarTest );

void GrammarTest::setUp() {
}

void GrammarTest::tearDown() {
}

void GrammarTest::stringParserTest() {
	{
		std::string input1 = 	"CNF (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ A -> A A | a,\n"
					"  B -> B B | b,\n"
					"  S -> | B S | S A},\n"
					"S)\n";
		grammar::Grammar grammar1 = factory::StringDataFactory::fromString (input1);
		std::string input2 = 	"CNF (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ A -> A A | a,\n"
					"  B -> B B | b,\n"
					"  S -> #E | B S | S A},\n"
					"S)\n";
		grammar::Grammar grammar2 = factory::StringDataFactory::fromString (input2);

		std::string output1 = factory::StringDataFactory::toString(grammar1);

		std::cout << "\"" << input1 << "\"" << std::endl << std::endl << "\"" << output1 << "\"" << std::endl;
		CPPUNIT_ASSERT( input1 == output1 );

		grammar::Grammar grammar3 = factory::StringDataFactory::fromString (output1);

		CPPUNIT_ASSERT( grammar1 == grammar3 );
		CPPUNIT_ASSERT( grammar1 == grammar2 );
	}
	{
		std::string input = 	"RIGHT_RG (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ A -> a | a A,\n"
					"  B -> b | b B,\n"
					"  S -> | a A | b A | b B},\n"
					"S)\n";
		grammar::Grammar grammar = factory::StringDataFactory::fromString (input);

		std::string output = factory::StringDataFactory::toString(grammar);

		std::cout << "\"" << input << "\"" << std::endl << std::endl << "\"" << output << "\"" << std::endl;
		CPPUNIT_ASSERT( input == output );

		grammar::Grammar grammar2 = factory::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
	{
		std::string input = 	"NON_CONTRACTING_GRAMMAR (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ A A -> B B | a A,\n"
					"  B -> b | b B,\n"
					"  S -> A S | B B | S A},\n"
					"S)\n";
		grammar::Grammar grammar = factory::StringDataFactory::fromString (input);

		std::string output = factory::StringDataFactory::toString(grammar);

		std::cout << "\"" << input << "\"" << std::endl << std::endl << "\"" << output << "\"" << std::endl;
		CPPUNIT_ASSERT( input == output );

		grammar::Grammar grammar2 = factory::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
	{
		std::string input = 	"CSG (\n"
					"{A, B, S},\n"
					"{a, b},\n"
					"{ | B | -> b | b B,\n"
					"  | S | -> A S | B B | S A,\n"
					"  A | A | -> B B | a A},\n"
					"S)\n";
		grammar::Grammar grammar = factory::StringDataFactory::fromString (input);

		std::string output = factory::StringDataFactory::toString(grammar);

		std::cout << "\"" << input << "\"" << std::endl << std::endl << "\"" << output << "\"" << std::endl;
		CPPUNIT_ASSERT( input == output );

		grammar::Grammar grammar2 = factory::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
	{
		grammar::RightRG < char > grammar ( 'S' );
		grammar.setTerminalAlphabet ( ext::set < char > { 'a', 'b' } );
		grammar.setGeneratesEpsilon ( true );
		grammar.addRule ( 'S', ext::make_pair ( 'a', 'S' ) );
		grammar.addRule ( 'S', 'b' );

		std::string output = factory::StringDataFactory::toString(grammar);
		std::cout << output << std::endl;

		std::string ref = "RIGHT_RG (\n"
				  "{S, T},\n"
				  "{a, b},\n"
				  "{ S -> a S | b,\n"
				  "  T -> | a S | b},\n"
				  "T)\n";

		CPPUNIT_ASSERT ( output == ref );
	}
}

