/*
 * astat.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Prints useful information about raw data structures", ' ', "0.01" );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < std::string > allowedPrintingOptions {
			"size"
		};
		TCLAP::ValuesConstraint < std::string > allowedPrintingOptionsVals ( allowedPrintingOptions );

		TCLAP::ValueArg < std::string > presentation ( "p", "presentation", "How to print data", true, "noop", & allowedPrintingOptionsVals );
		cmd.add ( presentation );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		TCLAP::ValueArg < std::string > input ( "i", "input", "Read from file", false, "-", "file" );
		cmd.add ( input );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stdin", input.getValue ( ) );
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		cli::Parser parser ( cli::Lexer ( "execute <#stdin > $input" ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		if ( presentation.getValue ( ) == "size" ) {
			parser = cli::Parser ( cli::Lexer ( "execute stats::SizeStat $input > $output" ) );
			parser.parse ( )->run ( environment );
		}

		measurements::end ( );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		parser = cli::Parser ( cli::Lexer ( "execute $output" ) );
		parser.parse ( )->run ( environment );

		measurements::end();
		measurements::end();

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		factory::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		common::Streams::out << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		common::Streams::err << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		common::Streams::err << "Unknown exception caught." << std::endl;
		return 127;
	}
}
